; pcsws.jss: IBM Personal Communications 3270 scripts
; Matt King, mattking@us.ibm.com
; version 1.24, 5/23/00

include"hjconst.jsh"
include "pcsws.jsh"
include "pcsws.jsm"

Function AutoStartEvent ()
let DeleteKeyPressed = FALSE
let BackspaceKeyPressed = FALSE
let DeleteWordKeyPressed = FALSE
let TerminalScreenEcho = ON
let StatusLineEcho = ON
Let MarginAlertCol = 70
let MarginAlertType = BELL
let MarginBell = GetJawsSettingsDirectory () + "\\PCSWSMarginBell.wav"
let InitializationSucceeded = SetWindowHandleVariables()
EndFunction

Script ScreenSensitiveHelp ()
if IsSameScript () then
	Run ( GetJawsSettingsDirectory () + "\\PCSWS.HTM")
	return
endIf
if InTerminalScreen() then
	Say (msg9, OT_HELP) ; This is a terminal emulation window.
	Say (msg10, OT_HELP) ; Use the standard navigation keys such as arrows, tab, back tab, home, page up and page down to navigate.
	Say (MSG11, OT_HELP) ; To learn about JAWS specific functions for this screen, press 
	SayString( GetScriptKeyName ( "HotKeyHelp"))
	Say (msg12, OT_HELP) ; or, to read all about all the script functions for this application, press 
	SayString( GetScriptKeyName ("ScreenSensitiveHelp")) 
	Say (msg8, OT_HELP) ; twice quickly
else
	PerformScript ScreenSensitiveHelp()
endIf
EndScript


Script HotKeyHelp ()
if InTerminalScreen() then
	Say (msg1, OT_HELP) ; To hear curren cursor position, press
	SayString( GetScriptKeyName ("SayActiveCursor"))
	Say (msg2, OT_HELP) ; To hear the bottom line of the emulation window, press
	SayString( GetScriptKeyName ("SayBottomLineOfEmulationWindow"))
	Say (msg3, OT_HELP) ; to hear the bottom two lines of the emulation window, press 
	SayString( GetScriptKeyName ("SayBottomTwoLinesOfEmulationWindow"))
	Say (msg4, OT_HELP) ; To hear the bottom three lines of the emulation window, press 
	SayString( GetScriptKeyName ("SayBottomThreeLinesOfEmulationWindow"))
	Say (msg5, OT_HELP) ; To read the emulation window starting from the top with the JAWS cursor, press
	SayString( GetScriptKeyName ("ReadEmulationWindow"))
	Say (msg6, OT_HELP) ; To hear the last nonblank line before the PC cursor, press 
	SayString( GetScriptKeyName ("ReadPreviousNonBlankLine"))
	Say (msg7, OT_HELP) ; To learn about other JAWS hot keys for this application, press 
	SayString( GetScriptKeyName ("ScreenSensitiveHelp"))
	Say (msg8, OT_HELP) ; twice quickly
	PerformScript HotKeyHelp()
else PerformScript HotKeyHelp()
endIf
EndScript

Script ProcessTabKey ()
var int RowNumber, int InTerminal
SaveCursor()
let InTerminal = InTerminalScreen()
if InTerminal then let RowNumber = getPCCursorRowNumber() endIf
PCCursor()
{tab}
pause()
if InTerminal then
   if RowNumber == getPCCursorRowNumber() then
      PerformScript sayWord()
   else
      PerformScript sayLine()
   endIf
endIf
RestoreCursor()
EndScript

Script ProcessBackTabKey ()
var int RowNumber, int InTerminal
SaveCursor()
let InTerminal = InTerminalScreen()
if InTerminal then let RowNumber = getPCCursorRowNumber() endIf
PCCursor()
{shift+tab}
pause()
if InTerminal then
   if RowNumber == getPCCursorRowNumber() then
      PerformScript sayWord()
   else
      PerformScript sayLine()
   endIf
endIf
RestoreCursor()
EndScript

Script ProcessShiftEnterKey ()
SaveCursor()
{shift+enter}
if InTerminalScreen() then
   pause()
   PCCursor ()
   PerformScript sayLine()
endIf
RestoreCursor()
EndScript

Script ProcessHomeKey ()
var int InTerminal
let InTerminal = InTerminalScreen()
if IsPCCursor () && InTerminal then
   {Home}
   pause()
   PerformScript sayLine()
else performScript JAWSHome()
endIf
EndScript

Script ProcessControlHomeKey ()
var int InTerminal
let InTerminal = InTerminalScreen()
if IsPCCursor () && InTerminal then
	{control+Home}
   pause()
   PerformScript sayLine()
else PerformScript TopOfFile()
endIf
EndScript

Script ProcessControlEndKey ()
var int InTerminal
let InTerminal = InTerminalScreen()
if IsPCCursor () && InTerminal then
	{control+end}
   pause()
   PerformScript sayLine()
else PerformScript BottomOfFile()
endIf
EndScript

Script ProcessExtendedAltDeleteKey ()
var int InTerminal
let InTerminal = InTerminalScreen()
if IsPCCursor () && InTerminal then
	{alt+delete}
   pause()
   PerformScript SayWord()
endIf
EndScript

Script JAWSDelete ()
PerformScript JAWSDelete()
EndScript

Script ReadPreviousNonBlankLine ()
var
	int NonBlankLineFound,
	int HitTopOfWindow,
	int CurrentRestriction,
	int PreviousRow
let NonBlankLineFound = FALSE
let HitTopOfWindow = FALSE
SaveCursor()
InvisibleCursor ()
RouteInvisibleToPc ()
let CurrentRestriction = GetRestriction ()
SetRestriction (RestrictWindow)
let PreviousRow = GetCursorRow ()
while ! NonBlankLineFound && ! HitTopOfWindow 
	PriorLine ()
	if GetCursorRow () == PreviousRow then let HitTopOfWindow = TRUE endIf 
	if strip( GetLine (), " ", BOTH) != "" then
		let NonBlankLineFound = TRUE
	else
		let PreviousRow = GetCursorRow ()
	endIf
endWhile
if HitTopOfWindow && ! NonBlankLineFound then
	Say (msg100, OT_STATUS) ; Already on first non blank line
ElIf HitTopOfWindow && NonBlankLineFound then
	say( msg101, OT_STATUS) ; Already on top line of Window
else
	SayLine ()
endIf
SetRestriction (CurrentRestriction)
RestoreCursor()
EndScript

Script SayPreviousNonBlankLineWithJAWSCursor ()
var
	int NonBlankLineFound,
	int HitTopOfWindow,
	int CurrentRestriction,
	int CurrentCursor,
	int PreviousRow
let NonBlankLineFound = FALSE
let HitTopOfWindow = FALSE
let CurrentCursor = GetActiveCursor ()
JAWSCursor ()
RouteJAWSToPc ()
let CurrentRestriction = GetRestriction ()
SetRestriction (RestrictWindow)
let PreviousRow = GetCursorRow ()
while ! NonBlankLineFound && ! HitTopOfWindow 
	PriorLine ()
	if GetCursorRow () == PreviousRow then let HitTopOfWindow = TRUE endIf 
	if strip( GetLine (), " ", BOTH) != "" then
		let NonBlankLineFound = TRUE
	else
		let PreviousRow = GetCursorRow ()
	endIf
endWhile
if HitTopOfWindow && ! NonBlankLineFound then
	Say (msg100, OT_STATUS) ; Already on first non blank line
	RouteJAWSToPc ()
	SetActiveCursor (CurrentCursor)
ElIf HitTopOfWindow && NonBlankLineFound then
	say( msg101, OT_STATUS) ; Already on top line of Window
	RouteJAWSToPc ()
	SetActiveCursor (CurrentCursor)
else
	SayLine ()
endIf
SetRestriction (CurrentRestriction)
EndScript

Script SayActiveCursor ()
var int InTerminal
let InTerminal = InTerminalScreen ()
if InTerminal && IsPCCursor() then 
   say( msg200, OT_STATUS) ; "row"
   SayInteger (GetPCCursorRowNumber())
   say( msg201, OT_STATUS) ; "Column "
   SayInteger (GetPCCursorColNumber())
else PerformScript SayActiveCursor()
endIf
EndScript

Script ReadEmulationWindow ()
MoveInvisibleToSyblingControl (TERMINAL_WINDOW_CONTROL_ID)
RouteJAWSToInvisible ()
JAWSCursor ()
SetRestriction( RestrictWindow)
SayAll ()
EndScript

Script SayBottomLineOfEmulationWindow ()
var int CurrentRestriction
SaveCursor()
if InTerminalScreen() then
	InvisibleCursor ()
	RouteInvisibleToPc ()
	let CurrentRestriction = GetRestriction ()
	SetRestriction (RestrictWindow)
	JAWSPageDown ()
	SayLine ()
	SetRestriction (CurrentRestriction)
else Say (msg102, OT_STATUS) ; "Not in terminal window"
endIf
RestoreCursor()
EndScript

Script SayBottomTwoLinesOfEmulationWindow ()
var int CurrentRestriction
SaveCursor()
if InTerminalScreen() then
	InvisibleCursor ()
	RouteInvisibleToPc ()
	let CurrentRestriction = GetRestriction ()
	SetRestriction (RestrictWindow)
	JAWSPageDown ()
	PriorLine ()
	SayLine ()
	NextLine ()
	SayLine ()
	SetRestriction (CurrentRestriction)
else Say (msg102, OT_STATUS) ; "Not in terminal window"
endIf
RestoreCursor()
EndScript

Script SayBottomThreeLinesOfEmulationWindow ()
var int CurrentRestriction
SaveCursor()
if InTerminalScreen() then
	InvisibleCursor ()
	RouteInvisibleToPc ()
	let CurrentRestriction = GetRestriction ()
	SetRestriction (RestrictWindow)
	JAWSPageDown ()
	PriorLine ()
	PriorLine ()
	SayLine ()
	NextLine ()
	SayLine ()
	NextLine ()
	SayLine ()
	SetRestriction (CurrentRestriction)
else Say (msg102, OT_STATUS) ; "Not in terminal window"
endIf
RestoreCursor()
EndScript

Script ReadFromSubjectLine ()
var
   int currentCursorRow,
   int currentCursorCol,
   int subjectLineFound
let subjectLineFound = FALSE
MoveInvisibleToSyblingControl (TERMINAL_WINDOW_CONTROL_ID)
RouteJAWSToInvisible ()
JAWSCursor()
let currentCursorRow = getCursorRow()
let currentCursorCol = getCursorCol()
FindString( getCurrentWindow(), "Subject:", S_BOTTOM, S_UNRESTRICTED)
if currentCursorRow != getCursorRow() | currentCursorCol != getCursorCol() then let subjectLineFound = TRUE endIf
if not subjectLineFound then
   FindString (getCurrentWindow(), "RE:", S_BOTTOM, S_UNRESTRICTED)
   if currentCursorRow != getCursorRow() | currentCursorCol != getCursorCol() then let subjectLineFound = TRUE endIf
endIf
if not subjectLineFound then say( msg103, OT_STATUS) ; "No subject line found"
else sayAll()
endIf
EndScript

Script SetEmulationWindowEcho ()
if TerminalScreenEcho then
	let TerminalScreenEcho = OFF
	Say( msg300, OT_STATUS) ; "Terminal Screen Echo off"
else
	let TerminalScreenEcho = ON
	Say (msg301, OT_STATUS) ; "Terminal Screen Echo on"
endIf
EndScript

Script SetStatusLineEcho ()
if StatusLineEcho == ON then
	let StatusLineEcho = OFF
	say( msg306, OT_STATUS) ; "System status echo off"
else
	let StatusLineEcho = ON
	say( msg307, OT_STATUS) ; "System Status Echo On"
endIf
EndScript

Script SetMarginAlert ()
If MarginAlertType == OFF then
	let MarginAlertType = BELL
	Say( msg302, OT_STATUS) ; "Margin bell on"
ElIF MarginAlertType == BELL then
	let MarginAlertType = PC_BEEP
	Say( msg303, OT_STATUS) ; "Margin beep on"
ElIf MarginAlertType == PC_BEEP then
	let MarginAlertType = OFF
	Say( msg304, OT_STATUS) ; "Margin alert off"
endIf
EndScript

Script IncrementMarginAlertColumn ()
let MarginAlertCol = MarginAlertCol + 1
sayInteger (MarginAlertCol)
say( msg305, OT_STATUS) ; " is margin alert position"
EndScript

Script DecrementMarginAlertColumn ()
let MarginAlertCol = MarginAlertCol - 1
sayInteger (MarginAlertCol)
say( msg305, OT_STATUS) ; " is margin alert position"
EndScript

Script  ScriptFileName()
ScriptAndAppNames( "Personal Communications Scripts Version 1.24 Dated 5/23/00") 
EndScript

Int Function GetPCCursorRowNumber ()
var
   int SlashPosition,
   string RowNumber
SaveCursor()
InvisibleCursor()
MoveToWindow (StatusLineHandle)
JAWSEnd ()
let RowNumber = GetWord ()
let SlashPosition = stringContains( RowNumber, "/")
if SlashPosition > 0 then let RowNumber = subString( RowNumber, 1, SlashPosition-1)
else let RowNumber = "0"
endIf
restoreCursor()
return StringToInt (RowNumber)
EndFunction

Int Function GetPCCursorColNumber ()
var
   int SlashPosition,
   string ColNumber
SaveCursor()
InvisibleCursor()
MoveToWindow (StatusLineHandle)
JAWSEnd ()
let ColNumber = GetWord ()
let SlashPosition = stringContains( ColNumber, "/")
if SlashPosition > 0 then let ColNumber = subString( ColNumber, SlashPosition+1, StringLength( ColNumber)-SlashPosition)
else let ColNumber = "0"
endIf
restoreCursor()
return StringToInt (ColNumber)
EndFunction

Int Function InTerminalScreen ()
var handle CurrentWindowHandle, int CurrentCursor
if MenusActive () == ACTIVE then return FALSE endIf
if ! InitializationSucceeded then
	let InitializationSucceeded = setWindowHandleVariables()
	if ! InitializationSucceeded then Return false endIf
endIf
let CurrentCursor = GetActiveCursor ()
InvisibleCursor ()
RouteInvisibleToPc ()
let CurrentWindowHandle = getCurrentWindow()
SetActiveCursor (CurrentCursor)
if CurrentWindowHandle == TERMINALWindowHandle then return TRUE
else return FALSE
endIf
EndFunction

Void Function NewTextEvent( handle hwnd, string buffer, int nAttributes, 
int nTextColor, int nBackgroundColor, int nEcho, string sFrameName)
var int InTerminal
let InTerminal = InTerminalScreen()
if DeleteKeyPressed then
   let DeleteKeyPressed = FALSE
   if InTerminal then return endIf
endIf
if BackspaceKeyPressed then
   let BackspaceKeyPressed = FALSE
   if InTerminal then return endIf
endIf
if DeleteWordKeyPressed then
   let DeleteWordKeyPressed = FALSE
	if InTerminal then return endIf
endIf
if hwnd == StatusLineHandle then 
	if StringContains( Buffer, "SYS") && StatusLineEcho then Say (buffer, ot_buffer) endIf
	if StringContains( Buffer, "\148") then SayString( "Insert Mode") endIf
endIf
if hwnd == TerminalWindowHandle then
	If InTerminal then
		if !TerminalScreenEcho then return endIf ; terminal screen is being silenced
		if (TerminalScreenEcho || (nAttributes & ATTRIB_HIGHLIGHT)) then
			; if Terminal echo is set to all or the text being written is highlighted
			Say (buffer, ot_buffer)
			return
		endif
	endIf
endIf
NewTextEvent( hwnd, buffer, nAttributes, nTextColor, nBackgroundColor, nEcho, sFrameName)
return
EndFunction

Void Function KeyPressedEvent (int nKey, string strKeyName, int nIsBrailleKey, int nIsScriptKey)
if nKey == 83 || nKey == 8275 then ; the delete key was pressed
   if InTerminalScreen() then let DeleteKeyPressed = TRUE endIf
ElIf nKey == 14 then ; BackspaceKey pressed
   if InTerminalScreen() then let BackSpaceKeyPressed = TRUE endIf
ElIf nKey == 8787 then ; alt+del pressed
	if InTerminalScreen() then let DeleteWordKeyPressed = TRUE endIf
endIf
EndFunction

Void Function FocusPointMovedEvent (int nX, int nY, int nOldX, int nOldY, int nUnit, int nDir, int nTimeElapsed)
var int CurrentCol, int InTerminal
if (nY == nOldY) && (MarginAlertType != OFF) then
	let InTerminal = InTerminalScreen()
	If InTerminal then
		let CurrentCol = GetPCCursorColNumber ()
		If CurrentCol == MarginAlertCol then
			if MarginAlertType == BELL then PlaySound ( MarginBell)
			else beep()
			endIf
		ElIf CurrentCol == (MarginAlertCol + 3) then
			if MarginAlertType == BELL then PlaySound ( MarginBell)
			else Beep ()
			endIf
		ElIf CurrentCol >= (MarginAlertCol + 6) then
			if MarginAlertType == BELL then PlaySound ( MarginBell)
			else Beep ()
			endIf
		endIf 
	endIf
endIf
EndFunction

Int Function SetWindowHandleVariables ()
Var int success, int CurrentCursor
let CurrentCursor = GetActiveCursor ()
let success = false
; if we get handles for both terminal and status windows, return true; otherwise return false.
if MoveInvisibleToSyblingControl ( TERMINAL_WINDOW_CONTROL_ID) then ; terminal window control ID found
	Let TerminalWindowHandle = GetCurrentWindow ()
	RouteInvisibleToPc () 
	; if control ID at PC cursor does not match Terminal Window Control ID,
	; then the terminal window is not yet bisible and we do not have the right handle.
	if  TerminalWindowHandle == GetCurrentWindow () then ; ID at cursor matches ID just found so ...
		if MoveInvisibleToSyblingControl ( STATUSLINE_CONTROL_ID) then ; status line window found
			let StatusLineHandle = GetCurrentWindow ()
			; now we have both handles.
			let success = TRUE
		else let success = FALSE ; status window not found
		endIf
	else let success = false ; Terminal Window Cursor ID mismatch
	endIf 
else let success = false ; terminal window not found
endIf
SetActiveCursor (CurrentCursor)
return success
EndFunction

Int Function MoveInvisibleToSyblingControl (int TargetControlID)
var handle theHandle
InvisibleCursor ()
RouteInvisibleToPc ()
let theHandle = FindDescendantWindow( GetParent( GetCurrentWindow()), TargetControlID)
if theHandle > 0 then ; supposedly found the control
   MoveToWindow( theHandle) ; move will fail if control not really found
   if GetControlID( GetCurrentWindow()) == TargetControlID then return TRUE
   else return FALSE
   endIf
else return FALSE
endIf
EndFunction

String Function strip (string stringToStrip, string charToStrip, int whereToStrip)
var
   int pos1, ; position of first or last character not to be stripped
   int lengthOfStringToStrip,
   string thisChar,
   int pos1Found
let lengthOfStringToStrip = stringLength( stringToStrip)
if (whereToStrip == LEADING | whereToStrip == BOTH) & lengthOfStringToStrip > 0 then
   let pos1Found = FALSE
   let pos1 = 1
   while (not pos1Found & pos1 <= lengthOfStringToStrip)
      let thisChar = substring( stringToStrip, pos1, 1)
      if( stringContains( thisChar, charToStrip) > 0 & pos1 == lengthOfStringToStrip)
      then ; the whole string is stripped down to a null
         let pos1Found = TRUE
         let pos1 = pos1 + 1 ; still necessary because we will test to see if pos1 > lengthOfStringToStrip
      elIf( stringContains( thisChar, charToStrip) > 0) then let pos1 = pos1 + 1
      else let pos1Found = TRUE
      endIf
   endWhile
   if pos1Found & pos1 > 1 then
      if pos1 > lengthOfStringToStrip then let stringToStrip = ""
      else let stringToStrip = subString( stringToStrip, pos1, (lengthOfStringToStrip - (pos1-1)))
      endIf
   endIf
endIf ; leading or both leading and trailing characters are to be stripped
let lengthOfStringToStrip = stringLength( stringToStrip)
if (whereToStrip == TRAILING | whereToStrip == BOTH) & lengthOfStringToStrip > 0 then
   let pos1Found = FALSE
   let pos1 = lengthOfStringToStrip
   while (not pos1Found & pos1 >= 1)
      let thisChar = subString( stringToStrip, pos1, 1)
      if( stringContains( thisChar, charToStrip) > 0 & pos1 == 1)
      then ; we will strip down to a null string
         let pos1Found = TRUE
         let pos1 = pos1 - 1 ; still necessary because we will test for pos1 = 0
      elIf( stringContains( thisChar, charToStrip) > 0) then let pos1 = pos1 - 1
      else let pos1Found = TRUE
      endIf
   endWhile
   if pos1Found & pos1 < lengthOfStringToStrip then
      if pos1 == 0 then let stringToStrip = ""
      else let stringToStrip = subString( stringToStrip, 1, pos1)
      endIf
   endIf
endIf ; trailing or both leading and trailing characters are to be stripped
return stringToStrip
EndFunction